"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const getStream = require("get-stream");
const path = require("path");
const exists = require("path-exists");
const pathTemp = require("path-temp");
const renameOverwrite = require("rename-overwrite");
const ssri = require("ssri");
const addFilesFromDir_1 = require("./addFilesFromDir");
const addFilesFromTarball_1 = require("./addFilesFromTarball");
const checkFilesIntegrity_1 = require("./checkFilesIntegrity");
exports.checkFilesIntegrity = checkFilesIntegrity_1.default;
const writeFile_1 = require("./writeFile");
function createCafs(cafsDir, ignore) {
    const locker = new Map();
    const _writeBufferToCafs = writeBufferToCafs.bind(null, locker, cafsDir);
    const addStream = addStreamToCafs.bind(null, _writeBufferToCafs);
    const addBuffer = addBufferToCafs.bind(null, _writeBufferToCafs);
    return {
        addFilesFromDir: addFilesFromDir_1.default.bind(null, { addBuffer, addStream }),
        addFilesFromTarball: addFilesFromTarball_1.default.bind(null, addStream, ignore !== null && ignore !== void 0 ? ignore : null),
    };
}
exports.default = createCafs;
async function addStreamToCafs(writeBufferToCafs, fileStream, mode) {
    const buffer = await getStream.buffer(fileStream);
    return addBufferToCafs(writeBufferToCafs, buffer, mode);
}
const modeIsExecutable = (mode) => (mode & 0o111) === 0o111;
async function addBufferToCafs(writeBufferToCafs, buffer, mode) {
    const integrity = ssri.fromData(buffer);
    const isExecutable = modeIsExecutable(mode);
    const fileDest = contentPathFromHex(isExecutable, integrity.hexDigest());
    await writeBufferToCafs(buffer, fileDest, isExecutable ? 0o755 : undefined);
    return integrity;
}
async function writeBufferToCafs(locker, cafsDir, buffer, fileDest, mode) {
    fileDest = path.join(cafsDir, fileDest);
    if (locker.has(fileDest)) {
        await locker.get(fileDest);
        return;
    }
    const p = (async () => {
        // This is a slow operation. Should be rewritten
        if (await exists(fileDest))
            return;
        // This might be too cautious.
        // The write is atomic, so in case pnpm crashes, no broken file
        // will be added to the store.
        // It might be a redundant step though, as we verify the contents of the
        // files before linking
        //
        // If we don't allow --no-verify-store-integrity then we probably can write
        // to the final file directly.
        const temp = pathTemp(path.dirname(fileDest));
        await writeFile_1.default(temp, buffer, mode);
        await renameOverwrite(temp, fileDest);
    })();
    locker.set(fileDest, p);
    await p;
}
function getFilePathInCafs(cafsDir, file) {
    return path.join(cafsDir, contentPathFromIntegrity(file.integrity, file.mode));
}
exports.getFilePathInCafs = getFilePathInCafs;
function contentPathFromIntegrity(integrity, mode) {
    const sri = ssri.parse(integrity, { single: true });
    const isExecutable = modeIsExecutable(mode);
    return contentPathFromHex(isExecutable, sri.hexDigest());
}
function contentPathFromHex(isExecutable, hex) {
    return (isExecutable ? `x${path.sep}` : '') +
        path.join(hex.slice(0, 2), hex.slice(2));
}
//# sourceMappingURL=index.js.map